//----------------------------------------------------------------
//
// OpenDCC
// Throttle MFT 1.0
//
// Copyright (c) 2008-2009 Wolfgang Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//-----------------------------------------------------------------
//
// file:      hardware.c
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// webpage:   http://www.opendcc.de
// history:   2008-12-08 V0.01 started
//
//-----------------------------------------------------------------
#include <inttypes.h>
#include <string.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "hardware.h"

static void init_port(void)
  {
    // Port A
    PORTA = (0<< LCD162_B0 )    // out (low)
          | (0<< LCD162_B1 )    // out (low)
          | (0<< LCD162_B2 )    // out
          | (0<< LCD162_B3 )    // out
          | (0<< LCD162_RS )    // out
          | (0<< LCD162_E  )    // out
          | (0<< KEY_SCAN1 )    // high = no scan (we scan on line1)
          | (1<< KEY_SCAN0 );   // high = no scan

    DDRA  = (1<< LCD162_B0 )    // out
          | (1<< LCD162_B1 )    // out
          | (1<< LCD162_B2 )    // out
          | (1<< LCD162_B3 )    // out
          | (1<< LCD162_RS )    // out
          | (1<< LCD162_E  )    // out
          | (1<< KEY_SCAN1 )    // out
          | (1<< KEY_SCAN0 );   // out

    // Port B
    PORTB = (1<< SPI_CS_EE )    // high = disable
          | (1<< SPI_SCK   )    // 
          | (1<< SPI_MOSI  )    // 
          | (1<< SPI_MISO  )    // pullup
          | (0<< LED_X0    )    // low = off
          | (0<< LED_X1    )    // low = off
          | (0<< LED_X2    )    // low = off
          | (0<< LED_X3    );   // low = off

    DDRB  = (1<< SPI_CS_EE )    // out
          | (1<< SPI_SCK   )    // out  / runs dedicated
          | (1<< SPI_MOSI  )    // out  / runs dedicated
          | (0<< SPI_MISO  )    // in   / runs dedicated
          | (1<< LED_X0    )    // out (red anode)
          | (1<< LED_X1    )    // out (red anode)
          | (1<< LED_X2    )    // out (red anode)
          | (1<< LED_X3    );   // out (red anode)

    // Port C
    PORTC = (1<< KEY_0     )    // pullup
          | (1<< KEY_1     )    // pullup
          | (1<< KEY_2     )    // pullup
          | (1<< KEY_3     )    // pullup
          | (1<< KEY_4     )    // pullup
          | (1<< KEY_5     )    // pullup / shift
          | (1<< KEY_6     )    // pullup / knob
          | (1<< KEY_7     );   // pullup / emergency stop

    DDRC  = (0<< KEY_0     )    // in
          | (0<< KEY_1     )    // in
          | (0<< KEY_2     )    // in
          | (0<< KEY_3     )    // in
          | (0<< KEY_4     )    // in
          | (0<< KEY_5     )    // in / shift
          | (0<< KEY_6     )    // in / knob
          | (0<< KEY_7     );   // in / emergency stop

    // Port D
    PORTD = (0<< MY_CTS    )    // out, disabled
          | (1<< MY_RTS    )    // in, pullup
          | (1<< MY_RXD    )    // in, pullup
          | (1<< MY_TXD    )    // out
          | (1<< ROT_A     )    // in, pullup
          | (0<< TXCAN     )    // out
          | (0<< RXCAN     )    // in
          | (1<< ROT_B     );   // in, pullup

    DDRD  = (1<< MY_CTS    )    // out
          | (0<< MY_RTS    )    // in
          | (0<< MY_RXD    )    // in
          | (1<< MY_TXD    )    // out
          | (0<< ROT_A     )    // in
          | (1<< TXCAN     )    // out
          | (0<< RXCAN     )    // in
          | (0<< ROT_B     );   // in

    // Port E
    PORTE = (1<< XP_RXD      )    // in
          | (0<< XP_TXD      )    // out
          | (1<< XP_RE       )    // out, high = disable
          | (0<< LED_XSTOP   )    // out (red anode), 0 = off
          | (0<< LED_YSTOP   )    // out (red cathode), 0 = off
          | (0<< LCD_PWM     )    // out, low = off
          | (1<< CAN_DISABL  )    // out
          | (0<< XP_DE       );   // out, low = disable

    DDRE  = (0<< XP_RXD      )    // in
          | (1<< XP_TXD      )    // out
          | (1<< XP_RE       )    // out
          | (1<< LED_XSTOP   )    // out (red anode)
          | (1<< LED_YSTOP   )    // out (red cathode)
          | (1<< LCD_PWM     )    // out
          | (1<< CAN_DISABL  )    // out
          | (1<< XP_DE       );   // out


    // Port F
    PORTF = (0<< VCCIO_SENSE )     // in analog (VCCIO)
          | (1<< POWER_ON    )     // out, high = turn on
          | (0<< VINT        )     // in analog (VINT/2)
          | (0<< A58         )     // in (reserved)
          | (0<< LED_Y0      )     // out (red cathode), low = off
          | (0<< LED_Y1      )     // out (red cathode), low = off
          | (0<< LED_Y2      )     // out (red cathode), low = off
          | (0<< LED_Y3      );    // out (red cathode), low = off

    DDRF  = (0<< VCCIO_SENSE )     // in analog (VCCIO)
          | (1<< POWER_ON    )     // out
          | (0<< VINT        )     // in analog (VINT/2)
          | (0<< A58         )     // in (reserved)
          | (1<< LED_Y0      )     // out (red cathode)
          | (1<< LED_Y1      )     // out (red cathode)
          | (1<< LED_Y2      )     // out (red cathode)
          | (1<< LED_Y3      );    // out (red cathode)


// Port G
    PORTG = (1<< XBEE_ENABL   )     // out (low = enabled)
          | (0<< XBEE_SLEEP   )     // out
          | (0<< SPI_ADR0     )     // out
          | (0<< FTDI_RESET   )     // out (low = disabled)
          | (1<< SPI_CS_LCD   );    // out

    DDRG  = (1<< XBEE_ENABL   )     // out (low = enabled)
          | (1<< XBEE_SLEEP   )     // out
          | (1<< SPI_ADR0     )     // out
          | (1<< FTDI_RESET   )     // out
          | (1<< SPI_CS_LCD   );    // out
  }


void init_timer(void)
  {
  }


/*    // Timer/Counter 0 initialization
    // Clock source: System Clock (10MHz)
    // Clock value: Timer 0 Stopped
    // Mode: Normal top=FFh
    // OC0 output: Disconnected
    ASSR=0x00;
    TCCR0=0x08;
    TCNT0=0x00;
    OCR0=0xE6;

    // Timer1: done in init_dccout();

    init_timer2();


    // Analog Comparator initialization
    // Analog Comparator: Off
    // Analog Comparator Input Capture by Timer/Counter 1: Off
    ACSR=0x80;
    SFIOR=0x00;   // PUD = 0: no "pull up disabled"

    invert_accessory = eeprom_read_byte((void *)eadr_invert_accessory);     // used for Lenz and XP
*/

static unsigned char bootloader_led;

void advance_bootloader_led(void)
  {
  	bootloader_led++;
  	bootloader_led &= 0x03;  // 8 states
  	switch (bootloader_led)
  	  {
  	  	case 0: LED_2_OFF; LED_3_GREEN; break;
  	  	case 1: LED_3_OFF; LED_0_GREEN; break;
  	  	case 2: LED_0_OFF; LED_1_GREEN; break;
  	  	case 3: LED_1_OFF; LED_2_GREEN; break;
  	  }
  }

void init_hardware(void)
  {
    init_port();
    init_timer();
    // init_interrupt();
  }


